package pl.ivend.xmlui;

import java.awt.Container;
import java.awt.MenuItem;
import java.io.IOException;
import java.io.InputStream;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import pl.ivend.xmlui.handler.HandlerMenu;
import pl.ivend.xmlui.handler.HandlerUI;

public final class FactoryUI {

	private static final String EXT_UI = ".ui.xml";
	private static final String EXT_MENU = ".menu.xml";

	private Object thisObject;
	private SAXParser parser;

	public FactoryUI(Object thisObject) {
		this.thisObject = thisObject;
		try {
			parser = SAXParserFactory.newInstance().newSAXParser();
		} catch (ParserConfigurationException e) {
			throw new RuntimeException(e);
		} catch (SAXException e) {
			throw new RuntimeException(e);
		}
	}

	/**
	 * @deprecated As of version 1.1, replaced by buildUI
	 * @see #buildUI(Container, String)
	 */
	@Deprecated
	public void newUI(Container rootComponentUI, String xmlDocument) {
		newUI(rootComponentUI,
				this.thisObject.getClass().getResourceAsStream(xmlDocument));
	}

	public void buildUI(Container rootComponentUI, String xmlDocument) {
		buildUI(rootComponentUI, this.thisObject.getClass()
				.getResourceAsStream(xmlDocument));
	}

	/**
	 * @deprecated As of version 1.1, replaced by buildUI
	 * @see #buildUI(MenuItem, String)
	 */
	@Deprecated
	public void newMenu(MenuItem rootComponentMenu, String xmlDocument) {
		newMenu(rootComponentMenu, this.thisObject.getClass()
				.getResourceAsStream(xmlDocument));
	}

	public void buildUI(MenuItem rootComponentMenu, String xmlDocument) {
		buildUI(rootComponentMenu, this.thisObject.getClass()
				.getResourceAsStream(xmlDocument));
	}

	private String initUI(String ext) {
		StringBuilder builder = new StringBuilder();
		builder.append(this.thisObject.getClass().getSimpleName());
		builder.append(ext);
		return builder.toString();
	}

	/**
	 * @deprecated As of version 1.1, replaced by buildUI
	 * @see #buildUI(Container)
	 */
	@Deprecated
	public void newUI(Container rootComponentUI) {
		newUI(rootComponentUI, initUI(EXT_UI));
	}

	public void buildUI(Container rootComponentUI) {
		buildUI(rootComponentUI, initUI(EXT_UI));
	}

	/**
	 * @deprecated As of version 1.1, replaced by buildUI
	 * @see #buildUI(MenuItem)
	 */
	@Deprecated
	public void newMenu(MenuItem rootComponentMenu) {
		newMenu(rootComponentMenu, initUI(EXT_MENU));
	}

	public void buildUI(MenuItem rootComponentMenu) {
		buildUI(rootComponentMenu, initUI(EXT_MENU));
	}

	/**
	 * @deprecated As of version 1.1, replaced by buildUI
	 * @see #buildUI(Container,InputStream)
	 */
	@Deprecated
	public void newUI(Container rootComponentUI, InputStream inputStream) {
		try {
			parser.parse(new InputSource(inputStream), new HandlerUI(
					this.thisObject, rootComponentUI));
		} catch (SAXException e) {
			throw new RuntimeException(e);
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
	}

	public void buildUI(Container rootComponentUI, InputStream inputStream) {
		try {
			parser.parse(new InputSource(inputStream), new HandlerUI(
					this.thisObject, rootComponentUI));
		} catch (SAXException e) {
			throw new RuntimeException(e);
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
	}

	/**
	 * @deprecated As of version 1.1, replaced by buildUI
	 * @see #buildUI(MenuItem,InputStream)
	 */
	@Deprecated
	public void newMenu(MenuItem rootComponentMenu, InputStream inputStream) {
		try {
			parser.parse(new InputSource(inputStream), new HandlerMenu(
					this.thisObject, rootComponentMenu));
		} catch (SAXException e) {
			throw new RuntimeException(e);
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
	}

	public void buildUI(MenuItem rootComponentMenu, InputStream inputStream) {
		try {
			parser.parse(new InputSource(inputStream), new HandlerMenu(
					this.thisObject, rootComponentMenu));
		} catch (SAXException e) {
			throw new RuntimeException(e);
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
	}

}
