package pl.ivend.xmlui.handler;

import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.apache.commons.beanutils.ConvertUtils;
import org.xml.sax.Attributes;
import org.xml.sax.helpers.DefaultHandler;

public abstract class AbstractHandler extends DefaultHandler {

	private static final String STRING = "java.lang.String";
	private static final String SEPARATOR_XMLNS = ":";
	private static final String SEPARATOR_CLASS = ".";
	private static final String METHOD_PREFIX = "set";

	private static final String BYTE = "byte";
	private static final String SHORT = "short";
	private static final String INT = "int";
	private static final String LONG = "long";
	private static final String CHAR = "char";
	private static final String FLOAT = "float";
	private static final String DOUBLE = "double";
	private static final String BOOLEAN = "boolean";
	private static final String VOID = "void";

	protected static final int LEVEL = 1;

	private Object thisObject;
	private boolean isNewNode;
	private int level;
	private String qName;
	private List<Attr> attrUIList;
	private Attributes attributes;
	private String swing;

	public AbstractHandler(Object thisObject, String xmlns) {
		this.thisObject = thisObject;
		this.level = 0;
		this.attrUIList = new ArrayList<Attr>();
		this.swing = xmlns;
	}

	protected class Attr {

		public String name;
		public String value;

		public Attr(String xml, String value) {
			this.name = xml;
			this.value = value;
		}

	}

	protected Object getThisObject() {
		return thisObject;
	}

	protected boolean isNewNode() {
		return isNewNode;
	}

	protected void setNewNode(boolean isNewNode) {
		this.isNewNode = isNewNode;
	}

	protected int getLevel() {
		return level;
	}

	protected void setLevel(int level) {
		this.level = level;
	}

	protected String getqName() {
		return qName;
	}

	protected void setqName(String qName) {
		this.qName = qName;
	}

	protected List<Attr> getAttrUIList() {
		return attrUIList;
	}

	protected void setAttrUIList(List<Attr> attrUIList) {
		this.attrUIList = attrUIList;
	}

	protected Attributes getAttributes() {
		return attributes;
	}

	protected void setAttributes(Attributes attributes) {
		this.attributes = attributes;
	}

	protected String getSwing() {
		return swing;
	}

	protected void setSwing(String swing) {
		this.swing = swing;
	}

	public Class<?> classForName(String name) {
		if (name.equals(BYTE))
			return byte.class;
		if (name.equals(SHORT))
			return short.class;
		if (name.equals(INT))
			return int.class;
		if (name.equals(LONG))
			return long.class;
		if (name.equals(CHAR))
			return char.class;
		if (name.equals(FLOAT))
			return float.class;
		if (name.equals(DOUBLE))
			return double.class;
		if (name.equals(BOOLEAN))
			return boolean.class;
		if (name.equals(VOID))
			return void.class;
		try {
			return Class.forName(name);
		} catch (ClassNotFoundException e) {
			return null;
		}
	}

	private String initSetField(String field) {
		StringBuilder builder = new StringBuilder();
		builder.append(METHOD_PREFIX);
		builder.append(field.substring(0, 1).toUpperCase());
		builder.append(field.substring(1));
		return builder.toString();
	}

	protected void initMethods(Object newNode, String... exclude) {
		for (int i = 0; i < getAttributes().getLength(); i++) {
			String field = getAttributes().getQName(i);
			if (!Arrays.asList(exclude).contains(field)) {
				Attr attr = initAttr(STRING, field);
				Class<?> clazz = classForName(attr.name);
				Object object = ConvertUtils.convert(getAttributes()
						.getValue(i), clazz);
				runMethod(newNode, initSetField(attr.value), clazz, object);
			}
		}
	}

	protected void initXmlns() {
		for (int i = 0; i < getAttributes().getLength(); i++) {
			String xmlns = getAttributes().getQName(i);
			String value = getAttributes().getValue(i);
			int indexOf = xmlns.indexOf(SEPARATOR_XMLNS);
			if (indexOf != -1) {
				String name = xmlns.substring(indexOf + 1);
				getAttrUIList().add(new Attr(name, value));
			} else {
				setSwing(value);
			}
		}
	}

	protected Attr initAttr(String swing, String name) {
		Attr attr = new Attr(swing, name);
		int indexOf = name.indexOf(SEPARATOR_XMLNS);
		if (indexOf != -1) {
			String xmlns = name.substring(0, indexOf);
			String value = name.substring(indexOf + 1);
			for (int i = 0; i < getAttrUIList().size(); i++) {
				String n = getAttrUIList().get(i).name;
				if (n.equals(xmlns)) {
					attr.name = getAttrUIList().get(i).value;
					attr.value = value;
					return attr;
				}
			}
		}
		return attr;
	}

	protected Class<?> initClazz(Attr attr) {
		StringBuilder builder = new StringBuilder();
		builder.append(attr.name);
		builder.append(SEPARATOR_CLASS);
		builder.append(attr.value);
		try {
			return Class.forName(builder.toString());
		} catch (ClassNotFoundException e) {
			return null;
		}
	}

	protected Object newObject(Class<?> clazz) {
		try {
			if (clazz == null) {
				return null;
			}
			return clazz.newInstance();
		} catch (InstantiationException e) {
			return null;
		} catch (IllegalAccessException e) {
			return null;
		}
	}

	protected void setField(Object o, Field field, Object value) {
		try {
			field.setAccessible(true);
			field.set(o, value);
		} catch (IllegalArgumentException e) {
		} catch (IllegalAccessException e) {
		}
	}

	protected void runMethod(Object object, String setField,
			Class<?> typeClass, Object typeValue) {
		try {
			Method method = object.getClass().getMethod(setField, typeClass);
			method.invoke(object, typeValue);
		} catch (SecurityException e) {
		} catch (NoSuchMethodException e) {
		} catch (IllegalArgumentException e) {
		} catch (IllegalAccessException e) {
		} catch (InvocationTargetException e) {
		}
	}

	protected abstract boolean isAnnotation(Field field, String name);

	protected Field getField(Class<?> clazz, String name) {
		if (name != null) {
			for (Field field : clazz.getDeclaredFields()) {
				if (isAnnotation(field, name)) {
					return field;
				}
			}
			if (clazz.getSuperclass() != null) {
				return getField(clazz.getSuperclass(), name);
			}
		}
		return null;
	}

	protected void initField(Object newNode, String nameObject) {
		Field field = getField(getThisObject().getClass(), getAttributes()
				.getValue(nameObject));
		if (field != null) {
			setField(getThisObject(), field, newNode);
		}
	}

	protected void initXmlns(String nameObject) {
		if (getqName().equals(nameObject)) {
			initXmlns();
		}
	}

}
