package pl.ivend.xmlui.handler;

import java.awt.Menu;
import java.awt.MenuContainer;
import java.awt.MenuItem;
import java.lang.reflect.Field;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

import pl.ivend.xmlui.annotation.XmlMenu;

public final class HandlerMenu extends AbstractHandler {

	private static final String AWT = "java.awt";
	private static final String MENU = "menu";

	private Menu componentMenu;
	private MenuItem componentCurrent;

	public HandlerMenu(Object thisObject, MenuItem rootComponentMenu) {
		super(thisObject, AWT);
		this.componentMenu = (Menu) rootComponentMenu;
	}

	@Override
	public void startElement(String uri, String localName, String qName,
			Attributes attributes) throws SAXException {
		setqName(qName);
		setAttributes(attributes);
		int level = getLevel();
		setLevel(++level);
		if (getLevel() == LEVEL) {
			initXmlns(MENU);
		} else {
			initNewNode();
		}
	}

	@Override
	public void endElement(String arg0, String arg1, String arg2)
			throws SAXException {
		if (isNewNode()) {
			if (componentMenu.equals(componentCurrent)) {
				MenuContainer parent = componentMenu.getParent();
				if (parent != null) {
					componentMenu = (Menu) parent;
				}
			}
			componentCurrent = componentMenu;
		}
	}

	@Override
	protected boolean isAnnotation(Field field, String name) {
		XmlMenu annotation = field.getAnnotation(XmlMenu.class);
		if ((annotation != null) && (field.getName().equals(name))) {
			return true;
		}
		return false;
	}

	private void initNewNode() {
		Class<?> clazz = initClazz(initAttr(getSwing(), getqName()));
		Object newNode = newObject(clazz);
		initField(newNode, MENU);
		initMethods(newNode, MENU);
		setNewNode(false);
		if (newNode instanceof MenuItem) {
			setNewNode(true);
			componentCurrent = (MenuItem) newNode;
			componentMenu.add(componentCurrent);
			if (newNode instanceof Menu) {
				componentMenu = (Menu) newNode;
			}
		}
	}

}
