package pl.ivend.xmlui.handler;

import java.awt.Component;
import java.awt.Container;
import java.awt.LayoutManager;
import java.lang.reflect.Field;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

import pl.ivend.xmlui.annotation.XmlUI;

public final class HandlerUI extends AbstractHandler {

	private static final String SWING = "javax.swing";
	private static final String UI = "ui";
	private static final String LAYOUT = "layout";
	private static final String ADD = "add";

	private Container componentUI;
	private Component componentCurrent;

	public HandlerUI(Object thisObject, Container rootComponentUI) {
		super(thisObject, SWING);
		this.componentUI = rootComponentUI;
	}

	@Override
	public void startElement(String uri, String localName, String qName,
			Attributes attributes) throws SAXException {
		setqName(qName);
		setAttributes(attributes);
		int level = getLevel();
		setLevel(++level);
		if (getLevel() == LEVEL) {
			initXmlns(UI);
		} else {
			initNewNode();
		}
	}

	@Override
	public void endElement(String arg0, String arg1, String arg2)
			throws SAXException {
		if (isNewNode()) {
			if (componentUI.equals(componentCurrent)) {
				Container parent = componentUI.getParent();
				if (parent != null) {
					componentUI = parent;
				}
			}
			componentCurrent = componentUI;
		}
	}

	@Override
	protected boolean isAnnotation(Field field, String name) {
		XmlUI annotation = field.getAnnotation(XmlUI.class);
		if ((annotation != null) && (field.getName().equals(name))) {
			return true;
		}
		return false;
	}

	private void setLayout(Container component, String className) {
		try {
			if (className != null) {
				Object object = newObject(Class.forName(className));
				if (object instanceof LayoutManager) {
					component.setLayout((LayoutManager) object);
				}
			}
		} catch (ClassNotFoundException e) {
			throw new RuntimeException(e);
		}
	}

	private void addElementToComponentUI(Container componentUI,
			Component componentCurrent, String constraints) {
		if (constraints != null) {
			componentUI.add(componentCurrent, constraints);
		} else {
			componentUI.add(componentCurrent);
		}
	}

	private void initNewNode() {
		Object newNode = newObject(initClazz(initAttr(getSwing(), getqName())));
		initField(newNode, UI);
		initMethods(newNode, UI, LAYOUT, ADD);
		setNewNode(false);
		if (newNode instanceof Component) {
			setNewNode(true);
			componentCurrent = (Component) newNode;
			if (newNode instanceof Container) {
				setLayout((Container) newNode, getAttributes().getValue(LAYOUT));
			}
			addElementToComponentUI(componentUI, componentCurrent,
					getAttributes().getValue(ADD));
			if (newNode instanceof Container) {
				componentUI = (Container) newNode;
			}
		}
	}

}
